@@ -7,11 +7,21 @@ import h5py
 from .utils import sanitize_array
 
 
+def bin_encode(array: Sequence[Number]) -> bytes:
+    """Sanitize an array and convert it to bytes.
+
+    :param array: Data to convert
+    """
+    sanitized_array = sanitize_array(array)
+
+    return sanitized_array.tobytes()
+
+
 def orjson_default(o) -> Union[list, str, None]:
     """Converts Python objects to JSON-serializable objects.
 
     :raises TypeError: if the object is not supported."""
-    if isinstance(o, np.generic) or isinstance(o, np.ndarray):
+    if isinstance(o, (np.generic, np.ndarray)):
         return o.tolist()
     if isinstance(o, complex):
         return [o.real, o.imag]
@@ -19,6 +29,8 @@ def orjson_default(o) -> Union[list, str, None]:
         return None
     if isinstance(o, bytes):
         return o.decode()
+    if isinstance(o, (h5py.RegionReference, h5py.Reference)):
+        return str(o)
     raise TypeError
 
 
@@ -66,7 +78,7 @@ def npy_stream(array: Sequence[Number]) -> Generator[bytes, None, None]:
 
 
 class Response(NamedTuple):
-    content: Generator[bytes, None, None]
+    content: Union[Generator[bytes, None, None], bytes]
     """ Encoded `content` as a generator of bytes """
     headers: Dict[str, str]
     """ Associated headers """
@@ -86,10 +98,11 @@ def encode(content, encoding: Optional[str] = "json") -> Response:
     """
     if encoding in ("json", None):
         return Response(
-            (chunk for chunk in (orjson_encode(content),)),  # generator
+            orjson_encode(content),
             headers={"Content-Type": "application/json"},
         )
-    elif encoding == "npy":
+
+    if encoding == "npy":
         return Response(
             npy_stream(content),
             headers={
@@ -97,5 +110,13 @@ def encode(content, encoding: Optional[str] = "json") -> Response:
                 "Content-Disposition": 'attachment; filename="data.npy"',
             },
         )
-    else:
-        raise ValueError(f"Unsupported encoding {encoding}")
+
+    if encoding == "bin":
+        return Response(
+            bin_encode(content),
+            headers={
+                "Content-Type": "application/octet-stream",
+            },
+        )
+
+    raise ValueError(f"Unsupported encoding {encoding}")
