@@ -1,5 +1,4 @@
 """Helpers for usage with `Flask <https://flask.palletsprojects.com/>`_"""
-from h5grove.utils import NotFoundError
 from flask import abort, Blueprint, current_app, request, Response, Request
 import h5py
 import os
@@ -7,6 +6,7 @@ from typing import Any, Callable, Mapping, Optional
 
 from .content import create_content, DatasetContent, ResolvedEntityContent
 from .encoders import encode
+from .utils import NotFoundError, parse_bool_arg
 
 
 __all__ = [
@@ -19,9 +19,9 @@ __all__ = [
 ]
 
 
-def make_encoded_response(content, format: Optional[str]) -> Response:
+def make_encoded_response(content, format_arg: Optional[str]) -> Response:
     """Prepare flask Response according to format"""
-    encoded_content, headers = encode(content, format)
+    encoded_content, headers = encode(content, format_arg)
     response = Response(encoded_content)
     response.headers.update(headers)
     return response
@@ -35,8 +35,8 @@ def get_content(h5file: h5py.File, path: Optional[str], resolve_links: bool = Tr
         abort(404, str(e))
 
 
-def get_filename(request: Request) -> str:
-    file_path = request.args.get("file")
+def get_filename(a_request: Request) -> str:
+    file_path = a_request.args.get("file")
     if file_path is None:
         raise KeyError("File argument is required")
 
@@ -51,12 +51,15 @@ def attr_route():
     """`/attr/` endpoint handler"""
     filename = get_filename(request)
     path = request.args.get("path")
-    format = request.args.get("format")
+    format_arg = request.args.get("format")
+    attr_keys = (
+        request.args.getlist("attr_keys") if "attr_keys" in request.args else None
+    )
 
     with h5py.File(filename, mode="r") as h5file:
         content = get_content(h5file, path)
         assert isinstance(content, ResolvedEntityContent)
-        return make_encoded_response(content.attributes(), format)
+        return make_encoded_response(content.attributes(attr_keys), format_arg)
 
 
 def data_route():
@@ -64,27 +67,25 @@ def data_route():
     filename = get_filename(request)
     path = request.args.get("path")
     selection = request.args.get("selection")
-    format = request.args.get("format")
+    format_arg = request.args.get("format")
+    flatten = parse_bool_arg(request.args.get("flatten"), fallback=False)
 
     with h5py.File(filename, mode="r") as h5file:
         content = get_content(h5file, path)
         assert isinstance(content, DatasetContent)
-        return make_encoded_response(content.data(selection), format)
+        return make_encoded_response(content.data(selection, flatten), format_arg)
 
 
 def meta_route():
     """`/meta/` endpoint handler"""
     filename = get_filename(request)
     path = request.args.get("path")
-    format = request.args.get("format")
-    resolve_links_arg = request.args.get("resolve_links")
-    resolve_links = (
-        resolve_links_arg.lower() != "false" if resolve_links_arg is not None else True
-    )
+    format_arg = request.args.get("format")
+    resolve_links = parse_bool_arg(request.args.get("resolve_links"), fallback=True)
 
     with h5py.File(filename, mode="r") as h5file:
         content = get_content(h5file, path, resolve_links)
-        return make_encoded_response(content.metadata(), format)
+        return make_encoded_response(content.metadata(), format_arg)
 
 
 def stats_route():
@@ -92,12 +93,12 @@ def stats_route():
     filename = get_filename(request)
     path = request.args.get("path")
     selection = request.args.get("selection")
-    format = request.args.get("format")
+    format_arg = request.args.get("format")
 
     with h5py.File(filename, mode="r") as h5file:
         content = get_content(h5file, path)
         assert isinstance(content, DatasetContent)
-        return make_encoded_response(content.data_stats(selection), format)
+        return make_encoded_response(content.data_stats(selection), format_arg)
 
 
 URL_RULES = {
